package controleur;

import java.time.LocalTime;

import domaine.Alarme;
import domaine.ListeDAlarmes;
import domaine.ResultatEnregistrementChargement;
import domaine.temps.Heure;
import static outils.NomDeFichier.*;
import vue.DialogueAlarme;
import vue.FenetreDAlarmes;

import javax.swing.*;
import javax.swing.filechooser.FileNameExtensionFilter;

public class Controleur extends Thread {
    private FenetreDAlarmes fenetre;
    private ListeDAlarmes alarmes;
    public Controleur() {
        fenetre = null;
        alarmes = null;
    }
    public void vue(FenetreDAlarmes f) {
        fenetre = f;
    }
    public void modele(ListeDAlarmes l) {
        alarmes = l;
    }
    private String messagePourErreurEnregistrementChargement(ResultatEnregistrementChargement resultat ) {
    	switch (resultat) {
			case FICHIER_INEXISTANT : return "Un problème est survenu à l'ouverture du fichier.";
			case ERREUR_DE_CREATION : return "Un problème est survenu à la création du fichier.";
	    	case ERREUR_DE_VERSION  : return "Cette application ne gère pas cette version de fichier.";
	    	case ERREUR_DE_LECTURE  : return "Un problème est survenu pendant la lecture du fichier.";
	    	case ERREUR_D_ECRITURE  : return "Un problème est survenu pendant l'écriture dans le fichier.";
    		default                 : return "";    			
    	}
    }
    public void charge() {
        JFileChooser fileChooser = new JFileChooser();
        fileChooser.setFileFilter(new FileNameExtensionFilter("Fichiers texte", "txt"));
        fileChooser.setAcceptAllFileFilterUsed(false);
        switch (fileChooser.showOpenDialog(fenetre)) {
            case JFileChooser.CANCEL_OPTION  :
                break;
            case JFileChooser.APPROVE_OPTION :
            	String nomDeFichier = fileChooser.getSelectedFile().getAbsolutePath();
            	if (!terminePar(nomDeFichier, ".txt"))
            		nomDeFichier += ".txt";
            	ResultatEnregistrementChargement resultat = alarmes.charge(nomDeFichier);
            	switch (resultat) {
	            	case REUSSI :
	                    fenetre.metAJourLaListe();
	                    fenetre.desactiveEditeur();
	                    fenetre.metAJourLeTitre();
	                    fenetre.activeOptionEnregistrer(true);
	                    break;
	                default :
	                    JOptionPane.showMessageDialog(fenetre,
	                    		messagePourErreurEnregistrementChargement(resultat) + "\nLes alarmes n'ont pas été chargées.",
	                            "Alarmes non chargées",
	                            JOptionPane.ERROR_MESSAGE);
            	}
            	break;
            case JFileChooser.ERROR_OPTION   :
                JOptionPane.showMessageDialog(fenetre,
                        "Un problème est survenu lors du choix de fichier.\nLes alarmes n'ont pas été chargées.",
                        "Alarmes non chargées",
                        JOptionPane.ERROR_MESSAGE);
                break;
        }
    }
    public void enregistre() {
    	ResultatEnregistrementChargement resultat = alarmes.enregistre();
    	switch (resultat) {
        	case REUSSI :
        		fenetre.metAJourLeTitre();
        		break;
        	default     :
	            JOptionPane.showMessageDialog(fenetre,
	            		messagePourErreurEnregistrementChargement(resultat),
	                    "Alarmes non enregistrées",
	                    JOptionPane.ERROR_MESSAGE);
        }
    }
    public void enregistreSous() {
        JFileChooser fileChooser = new JFileChooser();
        fileChooser.setFileFilter(new FileNameExtensionFilter("Fichiers texte", "txt"));
        fileChooser.setAcceptAllFileFilterUsed(false);
        switch (fileChooser.showSaveDialog(fenetre)) {
            case JFileChooser.CANCEL_OPTION  :
                break;
            case JFileChooser.APPROVE_OPTION :
            	String nomDeFichier = fileChooser.getSelectedFile().getAbsolutePath();
            	if (!terminePar(nomDeFichier, ".txt"))
            		nomDeFichier += ".txt";
            	ResultatEnregistrementChargement resultat = alarmes.enregistreSous(nomDeFichier);
            	switch (resultat) {
	            	case REUSSI :
	                    fenetre.metAJourLeTitre();
	                    fenetre.activeOptionEnregistrer(true);
	                    break;
	                default :
	                    JOptionPane.showMessageDialog(fenetre,
	                    		messagePourErreurEnregistrementChargement(resultat) + "\nLes alarmes n'ont pas été enregistrées.",
	                            "Alarmes non enregistrées",
	                            JOptionPane.ERROR_MESSAGE);
                }
                break;
            case JFileChooser.ERROR_OPTION   :
                JOptionPane.showMessageDialog(fenetre,
                        "Un problème est survenu lors du choix de fichier.\nLes alarmes n'ont pas été enregistrées.",
                        "Alarmes non enregistrées",
                        JOptionPane.ERROR_MESSAGE);
                break;
        }
    }
    public void nouvelleAlarme() {
        try {
            Alarme alarme = new Alarme("12:00");
            ajouteAlarme(alarme);
        } catch(Exception e) { };
    }
    public void ajouteAlarme(Alarme alarme) {
        alarmes.ajoute(alarme);
        alarmes.trie();
        alarmes.modifiee();
        fenetre.metAJourLaListe();
        fenetre.desactiveEditeur();
        fenetre.metAJourLeTitre();
    }
    public void modifieAlarme(Alarme alarme, String heure) {
        try { alarme.heure().affecteA(heure); } catch(Exception e) { }
        alarmes.trie();
        alarmes.modifiee();
        fenetre.metAJourLaListe();
        fenetre.desactiveEditeur();
        fenetre.metAJourLeTitre();
    }
    public void supprimeAlarme(Alarme alarme) {
        alarmes.retire(alarme);
        alarmes.modifiee();
        fenetre.metAJourLaListe();
        fenetre.desactiveEditeur();
        fenetre.metAJourLeTitre();
    }
    public void run() {
        while (true) {
            LocalTime now = LocalTime.now();
            for (Alarme a : alarmes) {
                Heure h = a.heure();
                if (h.heures() == now.getHour() && h.minutes() == now.getMinute()) {
                    DialogueAlarme.getInstance().affiche(a);
                    break;
                }
            }
            try { sleep(60000); } catch (InterruptedException e) { }
        }
    }
}
