package vue;

import java.awt.*;
import java.awt.event.ActionEvent;

import javax.swing.*;

import controleur.Controleur;
import controleur.EcouteurDHeure;
import controleur.EcouteurDeFin;
import controleur.EcouteurDeListeDAlarmes;
import controleur.EcouteurOuvrir;
import domaine.Alarme;
import domaine.ListeDAlarmes;

public class FenetreDAlarmes extends JFrame {
    private static final String FIN_TITRE_FENETRE = " - Alarmes";
    private Controleur controleur;
    private ListeDAlarmes lesAlarmes;
    private JList<Alarme> lstAlarmes;
    private JTextField txtHeureModifiée;
    private JButton btSupprime;
    private JButton btModifie;
    private BarreDeMenus menus;
    private BarreDEtat info;
    private Dimension tailleFenetre = new Dimension(500, 400);
    private Dimension tailleEtiquette = new Dimension(50, 20);
    private Dimension tailleHeure = new Dimension(40, 20);
    private Dimension tailleBouton = new Dimension(100, 25);
    private Dimension tailleEspace = new Dimension(10, 0);

    private void fixeTaille(Component composant, Dimension taille, boolean fixeMaximum) {
        composant.setMinimumSize(taille);
        if (fixeMaximum) composant.setMaximumSize(taille);
        composant.setPreferredSize(taille);
    }
    private JPanel panneauDEdition() {
        JPanel panneau = new JPanel();
        panneau.setLayout(new BoxLayout(panneau, BoxLayout.Y_AXIS));

        JPanel ligne1 = new JPanel();
        ligne1.setLayout(new FlowLayout(FlowLayout.LEFT));
        JLabel etiqHeure = new JLabel("Heure :");
        fixeTaille(etiqHeure, tailleEtiquette, true);
        ligne1.add(etiqHeure);
        txtHeureModifiée = new JTextField();
        fixeTaille(txtHeureModifiée, tailleHeure, true);
        txtHeureModifiée.addKeyListener(new EcouteurDHeure(txtHeureModifiée));
        ligne1.add(txtHeureModifiée);
        ligne1.add(Box.createRigidArea(tailleEspace));
        btModifie = new JButton("Modifier");
        fixeTaille(btModifie, tailleBouton, true);
        btModifie.addActionListener((ActionEvent e) -> {
            Alarme alarme = lstAlarmes.getSelectedValue();
            if (alarme != null) { controleur.modifieAlarme(alarme, txtHeureModifiée.getText()); }
        });
        ligne1.add(btModifie);
        ligne1.add(Box.createRigidArea(tailleEspace));
        btSupprime = new JButton("Supprimer");
        fixeTaille(btSupprime, tailleBouton, true);
        btSupprime.addActionListener((ActionEvent e) -> {
            Alarme alarme = lstAlarmes.getSelectedValue();
            if (alarme != null) { controleur.supprimeAlarme(alarme); }
        });
        ligne1.add(btSupprime);
        panneau.add(ligne1);

        return panneau;
    }
    public FenetreDAlarmes(ListeDAlarmes l, Controleur c) {
        fixeTaille(this, tailleFenetre, false);
    	lesAlarmes = l;
        controleur = c;
        metAJourLeTitre();
        menus = new BarreDeMenus(controleur,
        					     new EcouteurDeFin(this, lesAlarmes, "Quitter l'application", "Voulez-vous vraiment quitter l'applicaion ?"),
        					     new EcouteurOuvrir(this, lesAlarmes, controleur));
        setJMenuBar(menus);

        JPanel centre = new JPanel(new BorderLayout());
        lstAlarmes = new JList<Alarme>();
        lstAlarmes.addListSelectionListener(new EcouteurDeListeDAlarmes(this));
        JScrollPane defileurVertical = new JScrollPane(lstAlarmes);
        defileurVertical.setPreferredSize(new Dimension(250, 200));
        centre.add(defileurVertical, BorderLayout.CENTER);
        centre.add(panneauDEdition(), BorderLayout.SOUTH);

        add(centre, BorderLayout.CENTER);

        info = new BarreDEtat();
        add(info, BorderLayout.SOUTH);

        pack();
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        addWindowListener(new EcouteurDeFin(this, lesAlarmes, "Fermer l'application", "Voulez-vous vraiment fermer l'applicaion ?"));
        setLocationRelativeTo(null);
        
        desactiveEditeur();
    }
    public void activeOptionEnregistrer(boolean b) {
    	menus.activeOptionEnregistrer(b);
    }
    public void metAJourLaListe() {
        lstAlarmes.setListData(lesAlarmes.toArray(new Alarme[0]));
        info.nombreDAlarmesEst(lesAlarmes.nombreDElements());
    }
    public void metAJourLeTitre() {
        setTitle(lesAlarmes.nomAffichable() + FIN_TITRE_FENETRE);
    }
    public void desactiveEditeur() {
    	txtHeureModifiée.setText("");
    	txtHeureModifiée.setBackground(Color.WHITE);
    	txtHeureModifiée.setEnabled(false);
    	btSupprime.setEnabled(false);
    	btModifie.setEnabled(false);
    }
    public void editeSelection() {
    	Alarme a = lstAlarmes.getSelectedValue();
        if (a != null) {
	    	txtHeureModifiée.setText(a.heure().toString());
	    	txtHeureModifiée.setEnabled(true);
	    	btSupprime.setEnabled(true);
	    	btModifie.setEnabled(true);
        }
    }
    public void afficheAlarme(Alarme a) {

    }
}
